/**
 * The Paper Tape Project -- Font subproject
 * PaperTapeFont C Library
 *
 * (c) 2008 Sven Kppel
 *
 * This program is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General
 * Public License along with this program; if not, see
 * http://www.gnu.org/licenses/.
 *
 **/

#ifndef __PAPERTAPE_FONT_H__
#define __PAPERTAPE_FONT_H__

#include <stdlib.h>
#include <stdio.h>

typedef unsigned char   byte_t;

struct _papertape_font_char {
	char ascii_id;
	byte_t* bytes;
	int bytes_n;

	struct _papertape_font_char* next;
};

typedef struct _papertape_font_char PAPERTAPE_FONT_CHAR;
typedef struct _papertape_font_char PAPERTAPE_FONT;

#define PAPERTAPE_FONT_CHARACTER_SPACING_ASCII_ID  '\t'
static byte_t PAPERTAPE_FONT_CHARACTER_SPACING_DEFAULT_BYTES[] = { 0x00 };

#define PAPERTAPE_FONT_WRITE_FIRST_LINES \
	"# The Paper Tape Project -- PAPERTAPE_FONT file.\n" \
	"# The content of this file describes a paper tape font. You can edit it\n" \
	"# with a text editor. Lines like that ones are comments. They are not allowed\n" \
	"# in data binary lines.\n" \
	"# See http://dev.technikum29.de/svn/paper-tape-project/\n"

#define PAPERTAPE_FONT_WRITE_LOGICAL_1_CHARACTER   '+'

#define PAPERTAPE_FONT_SCHEMATICS_DEFAULT "abcdefghijklmnopqrstuvwxyz0123456789"

PAPERTAPE_FONT* papertape_font_new_from_file(FILE* fh);
//PAPERTAPE_FONT* papertape_font_new_from_string(const char* string);

// Read only functions
byte_t *papertape_font_get_label(PAPERTAPE_FONT* font, const char* string);
int papertape_font_string_is_printable(PAPERTAPE_FONT *font, const char* string);

// Write functions
int papertape_font_set_char(PAPERTAPE_FONT* font, char ascii_id, const unsigned char* bytes, int bytes_n);
#define papertape_font_set_char_spacing(font, bytes, bytes_n) \
	(papertape_font_set_char(font, PAPERTAPE_FONT_CHARACTER_SPACING_ASCII_ID, bytes, bytes_n))
int papertape_font_write_to_file(PAPERTAPE_FONT* font, FILE *fh);


#endif /* __PAPERTAPE_FONT_H__ */
